@@ -29,6 +29,9 @@
 
 #include <QClipboard>
 #include <QGuiApplication>
+#include <QScrollBar>
+
+#include <ktexteditor_utils.h>
 
 static QString toolsConfigDir()
 {
@@ -141,10 +144,15 @@ void KateExternalToolsPlugin::removeTools(const std::vector<KateExternalTool *>
             continue;
         }
 
-        QString configFile = KateExternalTool::configFileName(tool->name);
-        if (!configFile.isEmpty()) {
+        if (QString configFile = KateExternalTool::configFileName(tool->name); !configFile.isEmpty()) {
             QFile::remove(toolsConfigDir() + configFile);
         }
+
+        // remove old name variant, too
+        if (QString configFile = KateExternalTool::configFileNameOldStyleOnlyForRemove(tool->name); !configFile.isEmpty()) {
+            QFile::remove(toolsConfigDir() + configFile);
+        }
+
         delete tool;
     }
 
@@ -154,7 +162,7 @@ void KateExternalToolsPlugin::removeTools(const std::vector<KateExternalTool *>
     m_tools.erase(it, m_tools.end());
 }
 
-void KateExternalToolsPlugin::save(KateExternalTool *tool, const QString &oldName) const
+void KateExternalToolsPlugin::save(KateExternalTool *tool, const QString &oldName)
 {
     const QString name = KateExternalTool::configFileName(tool->name);
     KConfig config(toolsConfigDir() + name);
@@ -166,6 +174,10 @@ void KateExternalToolsPlugin::save(KateExternalTool *tool, const QString &oldNam
     if (!oldName.isEmpty()) {
         const QString oldFile = toolsConfigDir() + KateExternalTool::configFileName(oldName);
         QFile::remove(oldFile);
+
+        // remove old variant, too
+        const QString oldFile2 = toolsConfigDir() + KateExternalTool::configFileNameOldStyleOnlyForRemove(oldName);
+        QFile::remove(oldFile2);
     }
 }
 
@@ -232,24 +244,26 @@ QVector<KateExternalTool> KateExternalToolsPlugin::defaultTools() const
     return m_defaultTools;
 }
 
-void KateExternalToolsPlugin::runTool(const KateExternalTool &tool, KTextEditor::View *view)
+KateToolRunner *KateExternalToolsPlugin::runnerForTool(const KateExternalTool &tool, KTextEditor::View *view, bool executingSaveTrigger)
 {
     // expand the macros in command if any,
     // and construct a command with an absolute path
     auto mw = view->mainWindow();
 
     // save documents if requested
-    if (tool.saveMode == KateExternalTool::SaveMode::CurrentDocument) {
-        // only save if modified, to avoid unnecessary recompiles
-        if (view->document()->isModified()) {
-            view->document()->save();
-        }
-    } else if (tool.saveMode == KateExternalTool::SaveMode::AllDocuments) {
-        const auto guiClients = mw->guiFactory()->clients();
-        for (KXMLGUIClient *client : guiClients) {
-            if (QAction *a = client->actionCollection()->action(QStringLiteral("file_save_all"))) {
-                a->trigger();
-                break;
+    if (!executingSaveTrigger) {
+        if (tool.saveMode == KateExternalTool::SaveMode::CurrentDocument) {
+            // only save if modified, to avoid unnecessary recompiles
+            if (view->document()->isModified()) {
+                view->document()->save();
+            }
+        } else if (tool.saveMode == KateExternalTool::SaveMode::AllDocuments) {
+            const auto guiClients = mw->guiFactory()->clients();
+            for (KXMLGUIClient *client : guiClients) {
+                if (QAction *a = client->actionCollection()->action(QStringLiteral("file_save_all"))) {
+                    a->trigger();
+                    break;
+                }
             }
         }
     }
@@ -281,13 +295,25 @@ void KateExternalToolsPlugin::runTool(const KateExternalTool &tool, KTextEditor:
 
     // Allocate runner on heap such that it lives as long as the child
     // process is running and does not block the main thread.
-    auto runner = new KateToolRunner(std::move(copy), view, this);
+    return new KateToolRunner(std::move(copy), view, this);
+}
 
+void KateExternalToolsPlugin::runTool(const KateExternalTool &tool, KTextEditor::View *view, bool executingSaveTrigger)
+{
+    auto runner = runnerForTool(tool, view, executingSaveTrigger);
     // use QueuedConnection, since handleToolFinished deletes the runner
     connect(runner, &KateToolRunner::toolFinished, this, &KateExternalToolsPlugin::handleToolFinished, Qt::QueuedConnection);
     runner->run();
 }
 
+void KateExternalToolsPlugin::blockingRunTool(const KateExternalTool &tool, KTextEditor::View *view, bool executingSaveTrigger)
+{
+    auto runner = runnerForTool(tool, view, executingSaveTrigger);
+    connect(runner, &KateToolRunner::toolFinished, this, &KateExternalToolsPlugin::handleToolFinished);
+    runner->run();
+    runner->waitForFinished();
+}
+
 void KateExternalToolsPlugin::handleToolFinished(KateToolRunner *runner, int exitCode, bool crashed)
 {
     auto view = runner->view();
@@ -337,7 +363,14 @@ void KateExternalToolsPlugin::handleToolFinished(KateToolRunner *runner, int exi
         // updates-enabled trick: avoid some flicker
         const bool wereUpdatesEnabled = view->updatesEnabled();
         view->setUpdatesEnabled(false);
+
+        Utils::KateScrollBarRestorer scrollRestorer(view);
+
+        // Reload doc
         view->document()->documentReload();
+
+        scrollRestorer.restore();
+
         view->setUpdatesEnabled(wereUpdatesEnabled);
     }
 
