@@ -13,6 +13,7 @@
 #include <KTextEditor/Document>
 #include <KTextEditor/Editor>
 #include <KTextEditor/View>
+#include <ktexteditor_version.h>
 
 #include <KConfig>
 #include <KConfigGroup>
@@ -24,6 +25,7 @@
 #include <KXmlGuiWindow>
 #include <QBitmap>
 #include <QComboBox>
+#include <QListView>
 #include <QMenu>
 #include <QMessageBox>
 #include <QRegularExpression>
@@ -160,6 +162,7 @@ KateExternalToolServiceEditor::KateExternalToolServiceEditor(KateExternalTool *t
     ui.chkReload->setChecked(m_tool->reload);
     ui.cmbOutput->setCurrentIndex(static_cast<int>(m_tool->outputMode));
     ui.edtCommand->setText(m_tool->cmdname);
+    ui.cmbTrigger->setCurrentIndex((int)m_tool->trigger);
 
     static const QRegularExpressionValidator cmdLineValidator(QRegularExpression(QStringLiteral("[\\w-]*")));
     ui.edtCommand->setValidator(&cmdLineValidator);
@@ -180,6 +183,7 @@ KateExternalToolServiceEditor::KateExternalToolServiceEditor(KateExternalTool *t
             ui.chkReload->setChecked(t.reload);
             ui.cmbOutput->setCurrentIndex(static_cast<int>(t.outputMode));
             ui.edtCommand->setText(t.cmdname);
+            ui.cmbTrigger->setCurrentIndex(static_cast<int>(t.trigger));
         });
     }
 
@@ -194,6 +198,12 @@ void KateExternalToolServiceEditor::slotOKClicked()
         return;
     }
 
+    const bool hasTrigger = ui.cmbTrigger->currentIndex() != (int)KateExternalTool::Trigger::None;
+    if (hasTrigger && ui.edtMimeType->text().isEmpty()) {
+        QMessageBox::information(this, i18n("External Tool"), i18n("With 'Trigger' enabled, at least one mimetype needs to be specified."));
+        return;
+    }
+
     accept();
 }
 
@@ -206,6 +216,7 @@ void KateExternalToolServiceEditor::showMTDlg()
         ui.edtMimeType->setText(d.chooser()->mimeTypes().join(QStringLiteral(";")));
     }
 }
+
 // END KateExternalToolServiceEditor
 
 // BEGIN KateExternalToolsConfigWidget
@@ -346,6 +357,7 @@ bool KateExternalToolsConfigWidget::editTool(KateExternalTool *tool)
         tool->reload = editor.ui.chkReload->isChecked();
         tool->outputMode = static_cast<KateExternalTool::OutputMode>(editor.ui.cmbOutput->currentIndex());
         tool->cmdname = editor.ui.edtCommand->text().trimmed();
+        tool->trigger = static_cast<KateExternalTool::Trigger>(editor.ui.cmbTrigger->currentIndex());
 
         tool->executable = editor.ui.edtExecutable->text().trimmed();
         tool->hasexec = tool->checkExec();
