@@ -9,7 +9,9 @@
 #include "kateexternaltool.h"
 #include "ui_toolview.h"
 
+#include <KTextEditor/Application>
 #include <KTextEditor/Document>
+#include <KTextEditor/Editor>
 #include <KTextEditor/MainWindow>
 #include <KTextEditor/View>
 
@@ -22,6 +24,7 @@
 #include <KXMLGUIFactory>
 #include <QMenu>
 #include <QStandardPaths>
+#include <ktexteditor_version.h>
 
 #include <QFontDatabase>
 #include <QKeyEvent>
@@ -134,7 +137,7 @@ void KateExternalToolsMenuAction::updateActionState(KTextEditor::Document *activ
     for (QAction *action : actions) {
         if (action && action->data().value<KateExternalTool *>()) {
             auto tool = action->data().value<KateExternalTool *>();
-            action->setEnabled(tool->matchesMimetype(mimeType));
+            action->setEnabled(tool->matchesMimetype(mimeType) || tool->mimetypes.isEmpty());
         }
     }
 }
@@ -167,6 +170,8 @@ KateExternalToolsPluginView::KateExternalToolsPluginView(KTextEditor::MainWindow
 
     // ESC should close & hide ToolView
     connect(m_mainWindow, &KTextEditor::MainWindow::unhandledShortcutOverride, this, &KateExternalToolsPluginView::handleEsc);
+    connect(m_mainWindow, &KTextEditor::MainWindow::viewChanged, this, &KateExternalToolsPluginView::slotViewChanged);
+    slotViewChanged(m_mainWindow->activeView());
 }
 
 KateExternalToolsPluginView::~KateExternalToolsPluginView()
@@ -263,6 +268,63 @@ void KateExternalToolsPluginView::handleEsc(QEvent *event)
     }
 }
 
+void KateExternalToolsPluginView::slotViewChanged(KTextEditor::View *v)
+{
+    if (m_currentView) {
+        disconnect(m_currentView->document(), &KTextEditor::Document::documentSavedOrUploaded, this, &KateExternalToolsPluginView::onDocumentSaved);
+#if KTEXTEDITOR_VERSION >= QT_VERSION_CHECK(5, 91, 0)
+        disconnect(m_currentView->document(), &KTextEditor::Document::aboutToSave, this, &KateExternalToolsPluginView::onDocumentAboutToSave);
+#endif
+    }
+    m_currentView = v;
+
+    if (!m_currentView) {
+        return;
+    }
+
+    connect(v->document(), &KTextEditor::Document::documentSavedOrUploaded, this, &KateExternalToolsPluginView::onDocumentSaved, Qt::UniqueConnection);
+
+#if KTEXTEDITOR_VERSION >= QT_VERSION_CHECK(5, 91, 0)
+    connect(v->document(), &KTextEditor::Document::aboutToSave, this, &KateExternalToolsPluginView::onDocumentAboutToSave, Qt::UniqueConnection);
+#endif
+}
+
+void KateExternalToolsPluginView::onDocumentSaved(KTextEditor::Document *doc)
+{
+    // We only want to run this in the current active mainwindow
+    if (KTextEditor::Editor::instance()->application()->activeMainWindow() != m_mainWindow) {
+        return;
+    }
+
+    const auto tools = m_plugin->tools();
+    for (KateExternalTool *tool : tools) {
+        const bool hasSaveTrigger = tool->trigger == KateExternalTool::Trigger::AfterSave;
+        if (hasSaveTrigger && tool->matchesMimetype(doc->mimeType())) {
+            m_plugin->runTool(*tool, m_currentView, /*exec save trigger=*/true);
+        }
+    }
+}
+
+void KateExternalToolsPluginView::onDocumentAboutToSave(KTextEditor::Document *doc)
+{
+    // We only want to run this in the current active mainwindow
+    if (KTextEditor::Editor::instance()->application()->activeMainWindow() != m_mainWindow) {
+        return;
+    }
+
+#if KTEXTEDITOR_VERSION >= QT_VERSION_CHECK(5, 91, 0)
+    const auto tools = m_plugin->tools();
+    for (KateExternalTool *tool : tools) {
+        const bool hasSaveTrigger = tool->trigger == KateExternalTool::Trigger::BeforeSave;
+        if (hasSaveTrigger && tool->matchesMimetype(doc->mimeType())) {
+            m_plugin->blockingRunTool(*tool, m_currentView, /*exec save trigger=*/true);
+        }
+    }
+#else
+    Q_UNUSED(doc)
+#endif
+}
+
 // END KateExternalToolsPluginView
 
 // kate: space-indent on; indent-width 4; replace-tabs on;
