@@ -182,7 +182,7 @@ void KateFileTree::setModel(QAbstractItemModel *model)
     header()->resizeSection(1, minSize);
 
     // proxy never emits rowsMoved
-    connect(m_proxyModel, &QAbstractItemModel::rowsMoved, this, &KateFileTree::onRowsMoved);
+    connect(m_proxyModel->sourceModel(), &QAbstractItemModel::rowsMoved, this, &KateFileTree::onRowsMoved);
 }
 
 void KateFileTree::onRowsMoved(const QModelIndex &, int, int, const QModelIndex &destination, int row)
@@ -586,7 +586,7 @@ void KateFileTree::slotRenameFile()
     *sc = connect(job, &KIO::CopyJob::copyingDone, doc, success);
 
     if (!job->exec()) {
-        KMessageBox::sorry(this, i18n("File \"%1\" could not be moved to \"%2\"", oldFileUrl.toDisplayString(), newFileUrl.toDisplayString()));
+        KMessageBox::error(this, i18n("File \"%1\" could not be moved to \"%2\"", oldFileUrl.toDisplayString(), newFileUrl.toDisplayString()));
         doc->openUrl(oldFileUrl);
     }
 }
@@ -810,7 +810,7 @@ void KateFileTree::slotDocumentDelete()
     if (url.isValid()) {
         KIO::DeleteJob *job = KIO::del(url);
         if (!job->exec()) {
-            KMessageBox::sorry(this, i18n("File \"%1\" could not be deleted.", url.toDisplayString()));
+            KMessageBox::error(this, i18n("File \"%1\" could not be deleted.", url.toDisplayString()));
         }
     }
 }
