@@ -25,6 +25,7 @@
 #include <ktexteditor/editor.h>
 
 #include "katefiletreedebug.h"
+#include "ktexteditor_utils.h"
 
 static constexpr int MaxHistoryItems = 10;
 
@@ -443,7 +444,7 @@ void KateFileTreeModel::clearModel()
     // remove all items
     // can safely ignore documentClosed here
 
-    beginRemoveRows(QModelIndex(), 0, qMax(m_root->childCount() - 1, 0));
+    beginResetModel();
 
     delete m_root;
     m_root = new ProxyItemDir(QStringLiteral("m_root"), nullptr);
@@ -453,7 +454,7 @@ void KateFileTreeModel::clearModel()
     m_editHistory.clear();
     m_brushes.clear();
 
-    endRemoveRows();
+    endResetModel();
 }
 
 void KateFileTreeModel::connectDocument(const KTextEditor::Document *doc)
@@ -611,6 +612,11 @@ int KateFileTreeModel::rowCount(const QModelIndex &parent) const
         return m_root->childCount();
     }
 
+    // we only have children for column 0
+    if (parent.column() != 0) {
+        return 0;
+    }
+
     const ProxyItem *item = static_cast<ProxyItem *>(parent.internalPointer());
     if (!item) {
         return 0;
@@ -677,6 +683,11 @@ bool KateFileTreeModel::hasChildren(const QModelIndex &parent) const
         return m_root->childCount() > 0;
     }
 
+    // we only have children for column 0
+    if (parent.column() != 0) {
+        return false;
+    }
+
     const ProxyItem *item = static_cast<ProxyItem *>(parent.internalPointer());
     if (!item) {
         return false;
@@ -1297,23 +1308,10 @@ void KateFileTreeModel::updateItemPathAndHost(ProxyItem *item)
 void KateFileTreeModel::setupIcon(ProxyItem *item)
 {
     Q_ASSERT(item != nullptr);
+    Q_ASSERT(item->doc() != nullptr);
 
-    QString icon_name;
-
-    if (item->flag(ProxyItem::Modified)) {
-        icon_name = QStringLiteral("document-save");
-    } else {
-        const QUrl url(item->path());
-        icon_name = QMimeDatabase().mimeTypeForFile(url.path(), QMimeDatabase::MatchExtension).iconName();
-    }
-
-    QIcon icon = QIcon::fromTheme(icon_name);
-
-    if (item->flag(ProxyItem::ModifiedExternally) || item->flag(ProxyItem::DeletedExternally)) {
-        icon = KIconUtils::addOverlay(icon, QIcon(QLatin1String("emblem-important")), Qt::TopLeftCorner);
-    }
-
-    item->setIcon(icon);
+    // use common method as e.g. in tabbar, too
+    item->setIcon(Utils::iconForDocument(item->doc()));
 }
 
 void KateFileTreeModel::resetHistory()
