@@ -21,7 +21,7 @@
 
 #include <KXMLGUIClient>
 
-class KToolBar;
+class QToolBar;
 
 class KateFileTree;
 class KateFileTreeModel;
@@ -46,7 +46,14 @@ public:
 
     const KateFileTreePluginSettings &settings();
 
-    void applyConfig(bool shadingEnabled, const QColor &viewShade, const QColor &editShade, bool listMode, int sortRole, bool showFullPath, bool showToolbar, bool closeButton);
+    void applyConfig(bool shadingEnabled,
+                     const QColor &viewShade,
+                     const QColor &editShade,
+                     bool listMode,
+                     int sortRole,
+                     bool showFullPath,
+                     bool showToolbar,
+                     bool closeButton);
 
 public Q_SLOTS:
     void viewDestroyed(QObject *view);
@@ -104,7 +111,7 @@ protected:
 
 private:
     QWidget *m_toolView;
-    KToolBar *m_toolbar;
+    QToolBar *m_toolbar;
     KateFileTree *m_fileTree;
     KateFileTreeProxyModel *m_proxyModel;
     QLineEdit *m_filter;
