@@ -13,6 +13,7 @@
 #include <QDateTime>
 #include <QFontDatabase>
 #include <QLayout>
+#include <QLocale>
 #include <QStyle>
 #include <QTextEdit>
 
@@ -89,7 +90,7 @@ void TextOutputWidget::showSuccessMessage(const QString &message)
 
 void TextOutputWidget::writeMessage(const QString &msg)
 {
-    m_output->append(QStringLiteral("%1: %2\n").arg(QDateTime::currentDateTime().toString(Qt::SystemLocaleDate)).arg(msg));
+    m_output->append(QStringLiteral("%1: %2\n").arg(QLocale::system().toString(QDateTime::currentDateTime(), QLocale::ShortFormat), msg));
 
     raise();
 }
