@@ -7,16 +7,20 @@
 #include "lspclientplugin.h"
 #include "lspclientconfigpage.h"
 #include "lspclientpluginview.h"
+#include "lspclientservermanager.h"
 
 #include "lspclient_debug.h"
 
 #include <KConfigGroup>
 #include <KDirWatch>
+#include <KLocalizedString>
 #include <KPluginFactory>
 #include <KSharedConfig>
 
 #include <QDir>
+#include <QMessageBox>
 #include <QStandardPaths>
+#include <QTimer>
 
 static const QString CONFIG_LSPCLIENT{QStringLiteral("lspclient")};
 static const QString CONFIG_SYMBOL_DETAILS{QStringLiteral("SymbolDetails")};
@@ -40,28 +44,42 @@ static const QString CONFIG_SERVER_CONFIG{QStringLiteral("ServerConfiguration")}
 static const QString CONFIG_SEMANTIC_HIGHLIGHTING{QStringLiteral("SemanticHighlighting")};
 static const QString CONFIG_SIGNATURE_HELP{QStringLiteral("SignatureHelp")};
 static const QString CONFIG_AUTO_IMPORT{QStringLiteral("AutoImport")};
+static const QString CONFIG_ALLOWED_COMMANDS{QStringLiteral("AllowedServerCommandLines")};
+static const QString CONFIG_BLOCKED_COMMANDS{QStringLiteral("BlockedServerCommandLines")};
 
 K_PLUGIN_FACTORY_WITH_JSON(LSPClientPluginFactory, "lspclientplugin.json", registerPlugin<LSPClientPlugin>();)
 
+/**
+ * ensure we don't spam the user with debug output per-default
+ */
+static const bool debug = (qgetenv("LSPCLIENT_DEBUG") == QByteArray("1"));
+static QLoggingCategory::CategoryFilter oldCategoryFilter = nullptr;
+void myCategoryFilter(QLoggingCategory *category)
+{
+    // deactivate info and debug if not debug mode
+    if (qstrcmp(category->categoryName(), "katelspclientplugin") == 0) {
+        category->setEnabled(QtInfoMsg, debug);
+        category->setEnabled(QtDebugMsg, debug);
+    } else if (oldCategoryFilter) {
+        oldCategoryFilter(category);
+    }
+}
+
 LSPClientPlugin::LSPClientPlugin(QObject *parent, const QList<QVariant> &)
     : KTextEditor::Plugin(parent)
     , m_settingsPath(QStandardPaths::writableLocation(QStandardPaths::AppConfigLocation) + QStringLiteral("/lspclient"))
     , m_defaultConfigPath(QUrl::fromLocalFile(m_settingsPath + QStringLiteral("/settings.json")))
+    , m_debugMode(debug)
 {
     // ensure settings path exist, for e.g. local settings.json
     QDir().mkpath(m_settingsPath);
 
-    /**
-     * handle plugin verbosity
-     * the m_debugMode will be used to e.g. set debug level for started clangd, too
-     */
-    m_debugMode = (qgetenv("LSPCLIENT_DEBUG") == QByteArray("1"));
-    if (!m_debugMode) {
-        QLoggingCategory::setFilterRules(QStringLiteral("katelspclientplugin.debug=false\nkatelspclientplugin.info=false"));
-    } else {
-        QLoggingCategory::setFilterRules(QStringLiteral("katelspclientplugin.debug=true\nkatelspclientplugin.info=true"));
+    // ensure we don't spam the user with debug messages per default, do this just once
+    if (!oldCategoryFilter) {
+        oldCategoryFilter = QLoggingCategory::installFilter(myCategoryFilter);
     }
 
+    // apply our config
     readConfig();
 }
 
@@ -71,7 +89,10 @@ LSPClientPlugin::~LSPClientPlugin()
 
 QObject *LSPClientPlugin::createView(KTextEditor::MainWindow *mainWindow)
 {
-    return LSPClientPluginView::new_(this, mainWindow);
+    if (!m_serverManager) {
+        m_serverManager = LSPClientServerManager::new_(this);
+    }
+    return LSPClientPluginView::new_(this, mainWindow, m_serverManager);
 }
 
 int LSPClientPlugin::configPages() const
@@ -113,6 +134,17 @@ void LSPClientPlugin::readConfig()
     m_signatureHelp = config.readEntry(CONFIG_SIGNATURE_HELP, true);
     m_autoImport = config.readEntry(CONFIG_AUTO_IMPORT, true);
 
+    // read allow + block lists as two separate keys, let block always win
+    const auto allowed = config.readEntry(CONFIG_ALLOWED_COMMANDS, QStringList());
+    const auto blocked = config.readEntry(CONFIG_BLOCKED_COMMANDS, QStringList());
+    m_serverCommandLineToAllowedState.clear();
+    for (const auto &cmd : allowed) {
+        m_serverCommandLineToAllowedState[cmd] = true;
+    }
+    for (const auto &cmd : blocked) {
+        m_serverCommandLineToAllowedState[cmd] = false;
+    }
+
     Q_EMIT update();
 }
 
@@ -141,7 +173,80 @@ void LSPClientPlugin::writeConfig() const
     config.writeEntry(CONFIG_SIGNATURE_HELP, m_signatureHelp);
     config.writeEntry(CONFIG_AUTO_IMPORT, m_autoImport);
 
+    // write allow + block lists as two separate keys
+    QStringList allowed, blocked;
+    for (const auto &it : m_serverCommandLineToAllowedState) {
+        if (it.second) {
+            allowed.push_back(it.first);
+        } else {
+            blocked.push_back(it.first);
+        }
+    }
+    config.writeEntry(CONFIG_ALLOWED_COMMANDS, allowed);
+    config.writeEntry(CONFIG_BLOCKED_COMMANDS, blocked);
+
     Q_EMIT update();
 }
 
+bool LSPClientPlugin::isCommandLineAllowed(const QStringList &cmdline)
+{
+    // check our allow list
+    // if we already have stored some value, perfect, just use that one
+    const QString fullCommandLineString = cmdline.join(QStringLiteral(" "));
+    if (const auto it = m_serverCommandLineToAllowedState.find(fullCommandLineString); it != m_serverCommandLineToAllowedState.end()) {
+        return it->second;
+    }
+
+    // else: queue asking the user async, will ensure we don't have duplicate dialogs
+    // tell the launching it is forbidden ATM
+    QTimer::singleShot(0, this, [this, fullCommandLineString]() {
+        askForCommandLinePermission(fullCommandLineString);
+    });
+    return false;
+}
+
+void LSPClientPlugin::askForCommandLinePermission(const QString &fullCommandLineString)
+{
+    // did we already store a new result for the given command line? just use that
+    if (const auto it = m_serverCommandLineToAllowedState.find(fullCommandLineString); it != m_serverCommandLineToAllowedState.end()) {
+        // we need to trigger config update if the command is not blocked any longer
+        if (it->second) {
+            Q_EMIT update();
+        }
+        return;
+    }
+
+    // is this command line request already open? => skip the new request, dialog is already there
+    if (!m_currentActiveCommandLineDialogs.insert(fullCommandLineString).second) {
+        return;
+    }
+
+    // ask user if the start should be allowed
+    QPointer<QMessageBox> msgBox(new QMessageBox());
+    msgBox->setWindowTitle(i18n("LSP server start requested"));
+    msgBox->setTextFormat(Qt::RichText);
+    msgBox->setText(
+        i18n("Do you want the LSP server to be started?<br><br>The full command line is:<br><br><b>%1</b><br><br>The choice can be altered via the config page "
+             "of the plugin.",
+             fullCommandLineString.toHtmlEscaped()));
+    msgBox->setStandardButtons(QMessageBox::Yes | QMessageBox::No);
+    msgBox->setDefaultButton(QMessageBox::Yes);
+    const bool allowed = (msgBox->exec() == QMessageBox::Yes);
+
+    // store new configured value
+    m_serverCommandLineToAllowedState.emplace(fullCommandLineString, allowed);
+
+    // inform the user if it was forbidden! do this here to just emit this once
+    if (!allowed) {
+        Q_EMIT showMessage(KTextEditor::Message::Information,
+                           i18n("User permanently blocked start of: '%1'.\nUse the config page of the plugin to undo this block.", fullCommandLineString));
+    }
+
+    // purge dialog entry
+    m_currentActiveCommandLineDialogs.erase(fullCommandLineString);
+
+    // flush config, will trigger update()
+    writeConfig();
+}
+
 #include "lspclientplugin.moc"
