@@ -15,7 +15,7 @@
 #include <KConfigGroup>
 #include <KGuiItem>
 #include <KLocalizedString>
-#include <KMimeTypeTrader>
+#include <KParts/PartLoader>
 #include <KParts/ReadOnlyPart>
 #include <KPluginMetaData>
 #include <KService>
@@ -137,13 +137,41 @@ void PreviewWidget::setTextEditorView(KTextEditor::View *view)
     resetTextEditorView(m_previewedTextEditorDocument);
 }
 
+std::optional<KPluginMetaData> KTextEditorPreview::PreviewWidget::findPreviewPart(const QStringList mimeTypes)
+{
+    for (const auto &mimeType : qAsConst(mimeTypes)) {
+        const auto offers = KParts::PartLoader::partsForMimeType(mimeType);
+
+        if (offers.isEmpty()) {
+            continue;
+        }
+
+        const KPluginMetaData service = offers.first();
+        qCDebug(KTEPREVIEW) << "Found preferred kpart named" << service.name() << "with library" << service.fileName() << "for mimetype" << mimeType;
+
+        // no interest in kparts which also just display the text (like katepart itself)
+        // TODO: what about parts which also support importing plain text and turning into richer format
+        // and thus have it in their mimetypes list?
+        // could that perhaps be solved by introducing the concept of "native" and "imported" mimetypes?
+        // or making a distinction between source editors/viewers and final editors/viewers?
+        // latter would also help other source editors/viewers like a hexeditor, which "supports" any mimetype
+        if (service.mimeTypes().contains(QLatin1String("text/plain"))) {
+            qCDebug(KTEPREVIEW) << "Blindly discarding preferred kpart as it also supports text/plain, to avoid useless plain/text preview.";
+            continue;
+        }
+
+        return service;
+    }
+    return {};
+}
+
 void PreviewWidget::resetTextEditorView(KTextEditor::Document *document)
 {
     if (!isVisible() || m_previewedTextEditorDocument != document) {
         return;
     }
 
-    KService::Ptr service;
+    std::optional<KPluginMetaData> service;
 
     if (m_previewedTextEditorDocument) {
         // TODO: mimetype is not set for new documents which have not been saved yet.
@@ -155,33 +183,8 @@ void PreviewWidget::resetTextEditorView(KTextEditor::Document *document)
         // Also try to guess from the content, if the above fails.
         mimeTypes << m_previewedTextEditorDocument->mimeType();
 
-        for (const auto &mimeType : qAsConst(mimeTypes)) {
-            service = KMimeTypeTrader::self()->preferredService(mimeType, QStringLiteral("KParts/ReadOnlyPart"));
-            if (service) {
-                qCDebug(KTEPREVIEW) << "Found preferred kpart service named" << service->name() << "with library" << service->library() << "for mimetype"
-                                    << mimeType;
+        service = findPreviewPart(mimeTypes);
 
-                if (service->library().isEmpty()) {
-                    qCWarning(KTEPREVIEW) << "Discarding preferred kpart service due to empty library name:" << service->name();
-                    service.reset();
-                }
-
-                // no interest in kparts which also just display the text (like katepart itself)
-                // TODO: what about parts which also support importing plain text and turning into richer format
-                // and thus have it in their mimetypes list?
-                // could that perhaps be solved by introducing the concept of "native" and "imported" mimetypes?
-                // or making a distinction between source editors/viewers and final editors/viewers?
-                // latter would also help other source editors/viewers like a hexeditor, which "supports" any mimetype
-                if (service && service->mimeTypes().contains(QLatin1String("text/plain"))) {
-                    qCDebug(KTEPREVIEW) << "Blindly discarding preferred service as it also supports text/plain, to avoid useless plain/text preview.";
-                    service.reset();
-                }
-
-                if (service) {
-                    break;
-                }
-            }
-        }
         if (!service) {
             qCDebug(KTEPREVIEW) << "Found no preferred kpart service for mimetypes" << mimeTypes;
         }
@@ -202,7 +205,7 @@ void PreviewWidget::resetTextEditorView(KTextEditor::Document *document)
 
     // change of preview type?
     // TODO: find a better id than library?
-    const QString serviceId = service ? service->library() : QString();
+    const QString serviceId = service ? service->pluginId() : QString();
 
     if (serviceId != m_currentServiceId) {
         if (m_partView) {
@@ -213,7 +216,7 @@ void PreviewWidget::resetTextEditorView(KTextEditor::Document *document)
 
         if (service) {
             qCDebug(KTEPREVIEW) << "Creating new kpart service instance.";
-            m_partView = new KPartView(service, this);
+            m_partView = new KPartView(*service, this);
             const bool autoupdate = m_autoUpdateAction->isChecked();
             m_partView->setAutoUpdating(autoupdate);
             int index = addWidget(m_partView->widget());
