@@ -18,11 +18,7 @@
 
 #include <KConfigGroup>
 #include <KLocalizedString>
-
-#if KCOREADDONS_VERSION >= QT_VERSION_CHECK(5, 85, 0)
 #include <KNetworkMounts>
-#endif
-
 #include <KSharedConfig>
 
 #include <QCoreApplication>
@@ -115,6 +111,12 @@ KateProjectPlugin::KateProjectPlugin(QObject *parent, const QList<QVariant> &)
             Q_EMIT activateProject(projectToActivate);
         });
     }
+
+    /**
+     * forward to meta-object system friendly version
+     */
+    connect(this, &KateProjectPlugin::projectCreated, this, &KateProjectPlugin::projectAdded);
+    connect(this, &KateProjectPlugin::pluginViewProjectClosing, this, &KateProjectPlugin::projectRemoved);
 }
 
 KateProjectPlugin::~KateProjectPlugin()
@@ -239,13 +241,16 @@ KateProject *KateProjectPlugin::projectForDir(QDir dir, bool userSpecified)
     return nullptr;
 }
 
-bool KateProjectPlugin::closeProject(KateProject *project)
+void KateProjectPlugin::closeProject(KateProject *project)
 {
-    QVector<KTextEditor::Document *> projectDocuments;
-    const auto docs = KTextEditor::Editor::instance()->application()->documents();
-    for (auto doc : docs)
-        if (QUrl(project->baseDir()).isParentOf(doc->url().adjusted(QUrl::RemoveScheme)))
-            projectDocuments.push_back(doc);
+    // collect all documents we have mapped to the projects we want to close
+    // we can not delete projects that still have some mapping
+    QList<KTextEditor::Document *> projectDocuments;
+    for (const auto &it : m_document2Project) {
+        if (it.second == project) {
+            projectDocuments.append(it.first);
+        }
+    }
 
     // if we have some documents open for this project, ask if we want to close, else just do it
     if (!projectDocuments.isEmpty()) {
@@ -253,29 +258,44 @@ bool KateProjectPlugin::closeProject(KateProject *project)
         const QString title = i18n("Confirm project closing: %1", project->name());
         const QString text = i18n("Do you want to close the project %1 and the related %2 open documents?", project->name(), projectDocuments.size());
         if (QMessageBox::Yes != QMessageBox::question(window, title, text, QMessageBox::No | QMessageBox::Yes, QMessageBox::Yes)) {
-            return false;
+            return;
         }
 
-        for (auto doc : projectDocuments)
-            KTextEditor::Editor::instance()->application()->closeDocument(doc);
+        // best effort document closing, some might survive
+        KTextEditor::Editor::instance()->application()->closeDocuments(projectDocuments);
     }
 
-    Q_EMIT pluginViewProjectClosing(project);
-    if (m_projects.removeOne(project)) {
+    // check: did all documents of the project go away? if not we shall not close it
+    if (!projectHasOpenDocuments(project)) {
+        Q_EMIT pluginViewProjectClosing(project);
+        m_projects.removeOne(project);
         delete project;
-        return true;
     }
+}
 
+QList<QObject *> KateProjectPlugin::projectsObjects() const
+{
+    QList<QObject *> list;
+    for (auto &p : m_projects) {
+        list.push_back(p);
+    }
+    return list;
+}
+
+bool KateProjectPlugin::projectHasOpenDocuments(KateProject *project) const
+{
+    for (const auto &it : m_document2Project) {
+        if (it.second == project) {
+            return true;
+        }
+    }
     return false;
 }
 
 KateProject *KateProjectPlugin::projectForUrl(const QUrl &url)
 {
     if (url.isEmpty() || !url.isLocalFile()
-#if KCOREADDONS_VERSION >= QT_VERSION_CHECK(5, 85, 0)
-        || KNetworkMounts::self()->isOptionEnabledForPath(url.toLocalFile(), KNetworkMounts::MediumSideEffectsOptimizations)
-#endif
-    ) {
+        || KNetworkMounts::self()->isOptionEnabledForPath(url.toLocalFile(), KNetworkMounts::MediumSideEffectsOptimizations)) {
         return nullptr;
     }
 
@@ -292,28 +312,23 @@ void KateProjectPlugin::slotDocumentCreated(KTextEditor::Document *document)
 
 void KateProjectPlugin::slotDocumentDestroyed(QObject *document)
 {
-    if (KateProject *project = m_document2Project.value(document)) {
-        project->unregisterDocument(static_cast<KTextEditor::Document *>(document));
+    const auto it = m_document2Project.find(static_cast<KTextEditor::Document *>(document));
+    if (it == m_document2Project.end()) {
+        return;
     }
 
-    m_document2Project.remove(document);
+    it->second->unregisterDocument(static_cast<KTextEditor::Document *>(document));
+    m_document2Project.erase(it);
 }
 
 void KateProjectPlugin::slotDocumentUrlChanged(KTextEditor::Document *document)
 {
-    KateProject *project = projectForUrl(document->url());
+    // unregister from old mapping
+    slotDocumentDestroyed(document);
 
-    if (KateProject *project = m_document2Project.value(document)) {
-        project->unregisterDocument(document);
-    }
-
-    if (!project) {
-        m_document2Project.remove(document);
-    } else {
-        m_document2Project[document] = project;
-    }
-
-    if (KateProject *project = m_document2Project.value(document)) {
+    // register for new project, if any
+    if (KateProject *project = projectForUrl(document->url())) {
+        m_document2Project.emplace(document, project);
         project->registerDocument(document);
     }
 }
@@ -395,7 +410,6 @@ KateProject *KateProjectPlugin::createProjectForDirectory(const QDir &dir)
     return project;
 }
 
-
 KateProject *KateProjectPlugin::createProjectForDirectory(const QDir &dir, const QVariantMap &projectMap)
 {
     // check if we already have the needed project opened
@@ -510,6 +524,17 @@ void KateProjectPlugin::setMultiProject(bool completion, bool gotoSymbol)
     writeConfig();
 }
 
+void KateProjectPlugin::setRestoreProjectsForSession(bool enabled)
+{
+    m_restoreProjectsForSession = enabled;
+    writeConfig();
+}
+
+bool KateProjectPlugin::restoreProjectsForSession() const
+{
+    return m_restoreProjectsForSession;
+}
+
 void KateProjectPlugin::readConfig()
 {
     KConfigGroup config(KSharedConfig::openConfig(), "project");
@@ -530,6 +555,8 @@ void KateProjectPlugin::readConfig()
     m_singleClickAction = (ClickAction)config.readEntry("gitStatusSingleClick", (int)ClickAction::ShowDiff);
     m_doubleClickAction = (ClickAction)config.readEntry("gitStatusDoubleClick", (int)ClickAction::StageUnstage);
 
+    m_restoreProjectsForSession = config.readEntry("restoreProjectsForSessions", false);
+
     Q_EMIT configUpdated();
 }
 
@@ -566,6 +593,8 @@ void KateProjectPlugin::writeConfig()
     config.writeEntry("gitStatusSingleClick", (int)m_singleClickAction);
     config.writeEntry("gitStatusDoubleClick", (int)m_doubleClickAction);
 
+    config.writeEntry("restoreProjectsForSessions", m_restoreProjectsForSession);
+
     Q_EMIT configUpdated();
 }
 
@@ -621,47 +650,51 @@ void KateProjectPlugin::unregisterVariables()
 
 void KateProjectPlugin::readSessionConfig(const KConfigGroup &config)
 {
-    // de-serialize all open projects as list of json documents
-    const auto projectList = config.readEntry("projects", QStringList());
-    for (const auto &project : projectList) {
-        const QVariantMap sMap = QJsonDocument::fromJson(project.toUtf8()).toVariant().toMap();
+    // de-serialize all open projects as list of JSON documents if allowed
+    if (restoreProjectsForSession()) {
+        const auto projectList = config.readEntry("projects", QStringList());
+        for (const auto &project : projectList) {
+            const QVariantMap sMap = QJsonDocument::fromJson(project.toUtf8()).toVariant().toMap();
 
-        // valid file backed project?
-        if (const auto file = sMap[QStringLiteral("file")].toString(); !file.isEmpty()) {
-            createProjectForFileName(file);
-            continue;
+            // valid file backed project?
+            if (const auto file = sMap[QStringLiteral("file")].toString(); !file.isEmpty() && QFileInfo(file).exists()) {
+                createProjectForFileName(file);
+                continue;
+            }
+
+            // valid path + data project?
+            if (const auto path = sMap[QStringLiteral("path")].toString(); !path.isEmpty() && QFileInfo(path).exists()) {
+                createProjectForDirectory(QDir(path), sMap[QStringLiteral("data")].toMap());
+                continue;
+            }
+
+            // we might arrive here if invalid data is store, just ignore that, we just loose session data
         }
-
-        // valid path + data project?
-        if (const auto path = sMap[QStringLiteral("path")].toString(); !path.isEmpty()) {
-            createProjectForDirectory(QDir(path), sMap[QStringLiteral("data")].toMap());
-            continue;
-        }
-
-        // we might arrive here if invalid data is store, just ignore that, we just loose session data
     }
 }
 
 void KateProjectPlugin::writeSessionConfig(KConfigGroup &config)
 {
-    // serialize all open projects as list of json documents
+    // serialize all open projects as list of JSON documents if allowed, always write the list to not leave over old data forever
     QStringList projectList;
-    for (const auto project : projects()) {
-        QVariantMap sMap;
+    if (restoreProjectsForSession()) {
+        for (const auto project : projects()) {
+            QVariantMap sMap;
 
-        // for file backed stuff, we just remember the file
-        if (project->isFileBacked()) {
-            sMap[QStringLiteral("file")] = project->fileName();
+            // for file backed stuff, we just remember the file
+            if (project->isFileBacked()) {
+                sMap[QStringLiteral("file")] = project->fileName();
+            }
+
+            // otherwise we remember the data we generated purely in memory
+            else {
+                sMap[QStringLiteral("data")] = project->projectMap();
+                sMap[QStringLiteral("path")] = project->baseDir();
+            }
+
+            // encode as one-lines JSON string
+            projectList.push_back(QString::fromUtf8(QJsonDocument::fromVariant(QVariant(sMap)).toJson(QJsonDocument::Compact)));
         }
-
-        // otherwise we remember the data we generated purely in memory
-        else {
-            sMap[QStringLiteral("data")] = project->projectMap();
-            sMap[QStringLiteral("path")] = project->baseDir();
-        }
-
-        // encode as one-lines json string
-        projectList.push_back(QString::fromUtf8(QJsonDocument::fromVariant(QVariant(sMap)).toJson(QJsonDocument::Compact)));
     }
     config.writeEntry("projects", projectList);
 }
