@@ -8,6 +8,8 @@
 #ifndef _KATE_PROJECT_PLUGIN_H_
 #define _KATE_PROJECT_PLUGIN_H_
 
+#include <unordered_map>
+
 #include <QDir>
 #include <QFileSystemWatcher>
 #include <QThreadPool>
@@ -33,6 +35,7 @@ class KateProjectPlugin : public KTextEditor::Plugin, public KTextEditor::Sessio
 {
     Q_OBJECT
     Q_INTERFACES(KTextEditor::SessionConfigInterface)
+    Q_PROPERTY(QList<QObject *> projects READ projectsObjects)
 
 public:
     explicit KateProjectPlugin(QObject *parent = nullptr, const QList<QVariant> & = QList<QVariant>());
@@ -63,11 +66,11 @@ public:
     KateProject *projectForDir(QDir dir, bool userSpecified = false);
 
     /**
-     * Search and close project for given project, if possible.
-     * @param project to search matching project for closing
-     * @return true if successful
+     * Try to close the given project.
+     * Will ask if the files belonging to the project shall be closed, if not this will just do nothing.
+     * @param project project to close
      */
-    bool closeProject(KateProject *project);
+    void closeProject(KateProject *project);
 
     /**
      * Search and open project that contains given url, if possible.
@@ -87,6 +90,18 @@ public:
         return m_projects;
     }
 
+    /**
+     * As above, in different form for property access.
+     */
+    QList<QObject *> projectsObjects() const;
+
+    /**
+     * Has the given project open documents?
+     * @param project project to check open document for
+     * @return has the given project open documents
+     */
+    bool projectHasOpenDocuments(KateProject *project) const;
+
     /**
      * Get global code completion.
      * @return global completion object for KTextEditor::View
@@ -103,7 +118,8 @@ public:
      */
     KateProject *projectForDocument(KTextEditor::Document *document)
     {
-        return m_document2Project.value(document);
+        const auto it = m_document2Project.find(document);
+        return (it != m_document2Project.end()) ? it->second : nullptr;
     }
 
     void setAutoRepository(bool onGit, bool onSubversion, bool onMercurial, bool onFossil);
@@ -129,6 +145,9 @@ public:
     void setDoubleClickAction(ClickAction cb);
     ClickAction doubleClickAcion();
 
+    void setRestoreProjectsForSession(bool enabled);
+    bool restoreProjectsForSession() const;
+
     /**
      * filesystem watcher to keep track of all project files
      * and auto-reload
@@ -160,6 +179,12 @@ Q_SIGNALS:
      */
     void projectCreated(KateProject *project);
 
+    /**
+     * As above, but with adjusted naming and meta-object type friendly.
+     */
+    void projectRemoved(QObject *project);
+    void projectAdded(QObject *project);
+
     /**
      * Signal that plugin configuration changed
      */
@@ -230,7 +255,7 @@ private:
     /**
      * Mapping document => project
      */
-    QHash<QObject *, KateProject *> m_document2Project;
+    std::unordered_map<KTextEditor::Document *, KateProject *> m_document2Project;
 
     /**
      * Project completion
@@ -243,6 +268,9 @@ private:
     bool m_autoMercurial = true;
     bool m_autoFossil = true;
 
+    // restore projects on session loading?
+    bool m_restoreProjectsForSession = true;
+
     // indexing is expensive, default off
     bool m_indexEnabled = false;
     QUrl m_indexDirectory;
