@@ -34,7 +34,6 @@
 #include <QFileDialog>
 #include <QHBoxLayout>
 #include <QKeyEvent>
-#include <QTimer>
 #include <QVBoxLayout>
 
 #define PROJECTCLOSEICON "window-close"
@@ -107,7 +106,7 @@ KateProjectPluginView::KateProjectPluginView(KateProjectPlugin *plugin, KTextEdi
 
     m_stackedProjectViews = new QStackedWidget(m_toolView);
     m_stackedProjectInfoViews = new QStackedWidget(m_toolInfoView);
-    m_stackedgitViews = new QStackedWidget(m_gitToolView.get());
+    m_stackedGitViews = new QStackedWidget(m_gitToolView.get());
 
     connect(m_projectsCombo,
             static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
@@ -119,15 +118,19 @@ KateProjectPluginView::KateProjectPluginView(KateProjectPlugin *plugin, KTextEdi
     connect(m_projectsCombo, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &KateProjectPluginView::slotCurrentChanged);
     connect(m_reloadButton, &QToolButton::clicked, this, &KateProjectPluginView::slotProjectReload);
 
-    connect(m_closeProjectButton, &QToolButton::clicked, this, &KateProjectPluginView::slotProjectAboutToClose);
-    connect(m_plugin, &KateProjectPlugin::pluginViewProjectClosing, this, &KateProjectPluginView::slotProjectClose);
+    connect(m_closeProjectButton, &QToolButton::clicked, this, &KateProjectPluginView::slotCloseProject);
+    connect(m_plugin, &KateProjectPlugin::pluginViewProjectClosing, this, &KateProjectPluginView::slotHandleProjectClosing);
 
     connect(m_gitStatusRefreshButton, &QToolButton::clicked, this, [this] {
-        if (auto widget = m_stackedgitViews->currentWidget()) {
-            qobject_cast<GitWidget *>(widget)->getStatus();
+        if (auto widget = m_stackedGitViews->currentWidget()) {
+            qobject_cast<GitWidget *>(widget)->updateStatus();
         }
     });
 
+    connect(&m_gitChangedWatcher, &QFileSystemWatcher::fileChanged, this, [this] {
+        slotUpdateStatus(true);
+    });
+
     /**
      * create views for all already existing projects
      * will create toolviews on demand!
@@ -175,18 +178,33 @@ KateProjectPluginView::KateProjectPluginView(KateProjectPlugin *plugin, KTextEdi
     a->setText(i18n("Project TODOs"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("korg-todo")));
 
-    m_projectPrevAction = a = actionCollection()->addAction(KStandardAction::Back, QStringLiteral("projects_prev_project"), this, SLOT(slotProjectPrev()));
+    m_projectPrevAction = a = actionCollection()->addAction(QStringLiteral("projects_prev_project"), this, SLOT(slotProjectPrev()));
+    a->setText(i18n("Activate Previous Project"));
+    a->setIcon(QIcon::fromTheme(QStringLiteral("arrow-left")));
     actionCollection()->setDefaultShortcut(a, QKeySequence(Qt::CTRL | Qt::ALT | Qt::Key_Left));
 
-    m_projectNextAction = a = actionCollection()->addAction(KStandardAction::Forward, QStringLiteral("projects_next_project"), this, SLOT(slotProjectNext()));
+    m_projectNextAction = a = actionCollection()->addAction(QStringLiteral("projects_next_project"), this, SLOT(slotProjectNext()));
+    a->setText(i18n("Activate Next Project"));
+    a->setIcon(QIcon::fromTheme(QStringLiteral("arrow-right")));
     actionCollection()->setDefaultShortcut(a, QKeySequence(Qt::CTRL | Qt::ALT | Qt::Key_Right));
 
     m_projectGotoIndexAction = a = actionCollection()->addAction(QStringLiteral("projects_goto_index"), this, SLOT(slotProjectIndex()));
     a->setText(i18n("Lookup"));
     actionCollection()->setDefaultShortcut(a, QKeySequence(Qt::ALT | Qt::Key_1));
-    m_projectCloseAction = a = actionCollection()->addAction(QStringLiteral("projects_close"), this, SLOT(slotProjectAboutToClose()));
+
+    m_projectCloseAction = a = actionCollection()->addAction(QStringLiteral("projects_close"), this, SLOT(slotCloseProject()));
     a->setText(i18n("Close Project"));
     a->setIcon(QIcon::fromTheme(QStringLiteral(PROJECTCLOSEICON)));
+
+    m_projectCloseAllAction = a = actionCollection()->addAction(QStringLiteral("projects_close_all"), this, SLOT(slotCloseAllProjects()));
+    a->setText(i18n("Close All Projects"));
+    a->setIcon(QIcon::fromTheme(QStringLiteral(PROJECTCLOSEICON)));
+
+    m_projectCloseWithoutDocumentsAction = a =
+        actionCollection()->addAction(QStringLiteral("projects_close_without_open_documents"), this, SLOT(slotCloseAllProjectsWithoutDocuments()));
+    a->setText(i18n("Close Orphaned Projects"));
+    a->setIcon(QIcon::fromTheme(QStringLiteral(PROJECTCLOSEICON)));
+
     m_gotoSymbolActionAppMenu = a = actionCollection()->addAction(KStandardAction::Goto, QStringLiteral("projects_goto_symbol"), this, SLOT(slotGotoSymbol()));
 
     // popup menu
@@ -223,6 +241,11 @@ KateProjectPluginView::KateProjectPluginView(KateProjectPlugin *plugin, KTextEdi
      */
     connect(this, &KateProjectPluginView::projectMapChanged, this, &KateProjectPluginView::updateActions);
     updateActions();
+
+    /**
+     * ensure we react on requests to active a project from the plugin
+     */
+    connect(m_plugin, &KateProjectPlugin::activateProject, this, &KateProjectPluginView::slotActivateProject);
 }
 
 KateProjectPluginView::~KateProjectPluginView()
@@ -300,7 +323,7 @@ QPair<KateProjectView *, KateProjectInfoView *> KateProjectPluginView::viewForPr
      */
     m_stackedProjectViews->addWidget(view);
     m_stackedProjectInfoViews->addWidget(infoView);
-    m_stackedgitViews->addWidget(gitView);
+    m_stackedGitViews->addWidget(gitView);
     m_projectsCombo->addItem(QIcon::fromTheme(QStringLiteral("project-open")), project->name(), project->fileName());
     m_projectsComboGit->addItem(QIcon::fromTheme(QStringLiteral("project-open")), project->name(), project->fileName());
 
@@ -420,7 +443,8 @@ void KateProjectPluginView::slotViewChanged()
      * update pointer, maybe disconnect before
      */
     if (m_activeTextEditorView) {
-        m_activeTextEditorView->document()->disconnect(this);
+        // but only url changed
+        disconnect(m_activeTextEditorView->document(), &KTextEditor::Document::documentUrlChanged, this, &KateProjectPluginView::slotDocumentUrlChanged);
     }
     m_activeTextEditorView = activeView;
 
@@ -436,18 +460,32 @@ void KateProjectPluginView::slotViewChanged()
      */
     connect(m_activeTextEditorView->document(), &KTextEditor::Document::documentUrlChanged, this, &KateProjectPluginView::slotDocumentUrlChanged);
 
+    /**
+     * Watch any document, as long as we live, if it's saved
+     */
+    connect(m_activeTextEditorView->document(),
+            &KTextEditor::Document::documentSavedOrUploaded,
+            this,
+            &KateProjectPluginView::slotDocumentSaved,
+            Qt::UniqueConnection);
+
     /**
      * trigger slot once
      */
     slotDocumentUrlChanged(m_activeTextEditorView->document());
 }
 
+void KateProjectPluginView::slotDocumentSaved()
+{
+    slotUpdateStatus(true);
+}
+
 void KateProjectPluginView::slotCurrentChanged(int index)
 {
     // trigger change of stacked widgets
     m_stackedProjectViews->setCurrentIndex(index);
     m_stackedProjectInfoViews->setCurrentIndex(index);
-    m_stackedgitViews->setCurrentIndex(index);
+    m_stackedGitViews->setCurrentIndex(index);
 
     {
         const QSignalBlocker blocker(m_projectsComboGit);
@@ -466,11 +504,17 @@ void KateProjectPluginView::slotCurrentChanged(int index)
     }
 
     // update git focus proxy + update status
-    if (QWidget *current = m_stackedgitViews->currentWidget()) {
-        m_stackedgitViews->setFocusProxy(current);
-        static_cast<GitWidget *>(current)->getStatus();
+    if (QWidget *current = m_stackedGitViews->currentWidget()) {
+        m_stackedGitViews->setFocusProxy(current);
     }
 
+    // Don't watch what nobody use, the old project...
+    if (!m_gitChangedWatcher.files().isEmpty()) {
+        m_gitChangedWatcher.removePaths(m_gitChangedWatcher.files());
+    }
+    // ...and start watching the new one
+    slotUpdateStatus(true);
+
     // project file name might have changed
     Q_EMIT projectFileNameChanged();
     Q_EMIT projectMapChanged();
@@ -600,19 +644,41 @@ void KateProjectPluginView::slotProjectReload()
     /**
      * Refresh git status
      */
-    if (auto widget = m_stackedgitViews->currentWidget()) {
-        qobject_cast<GitWidget *>(widget)->getStatus();
+    if (auto widget = m_stackedGitViews->currentWidget()) {
+        qobject_cast<GitWidget *>(widget)->updateStatus();
     }
 }
 
-void KateProjectPluginView::slotProjectAboutToClose()
+void KateProjectPluginView::slotCloseProject()
 {
     if (QWidget *current = m_stackedProjectViews->currentWidget()) {
         m_plugin->closeProject(static_cast<KateProjectView *>(current)->project());
     }
 }
 
-void KateProjectPluginView::slotProjectClose(KateProject *project)
+void KateProjectPluginView::slotCloseAllProjects()
+{
+    // we must close project after project
+    // project closing might activate a different one and then would load the files of that project again
+    const auto copiedProjects = m_plugin->projects();
+    for (auto project : copiedProjects) {
+        m_plugin->closeProject(project);
+    }
+}
+
+void KateProjectPluginView::slotCloseAllProjectsWithoutDocuments()
+{
+    // we must close project after project
+    // project closing might activate a different one and then would load the files of that project again
+    const auto copiedProjects = m_plugin->projects();
+    for (auto project : copiedProjects) {
+        if (!m_plugin->projectHasOpenDocuments(project)) {
+            m_plugin->closeProject(project);
+        }
+    }
+}
+
+void KateProjectPluginView::slotHandleProjectClosing(KateProject *project)
 {
     const int index = m_plugin->projects().indexOf(project);
     m_project2View.erase(m_project2View.find(project));
@@ -625,15 +691,22 @@ void KateProjectPluginView::slotProjectClose(KateProject *project)
     m_stackedProjectInfoViews->removeWidget(stackedProjectInfoViewsWidget);
     delete stackedProjectInfoViewsWidget;
 
-    QWidget *stackedgitViewsWidget = m_stackedgitViews->widget(index);
-    m_stackedgitViews->removeWidget(stackedgitViewsWidget);
+    QWidget *stackedgitViewsWidget = m_stackedGitViews->widget(index);
+    m_stackedGitViews->removeWidget(stackedgitViewsWidget);
     delete stackedgitViewsWidget;
 
     m_projectsCombo->removeItem(index);
     m_projectsComboGit->removeItem(index);
 
+    // Stop watching what no one is interesting anymore
+    if (!m_gitChangedWatcher.files().isEmpty()) {
+        m_gitChangedWatcher.removePaths(m_gitChangedWatcher.files());
+    }
     // inform onward
     Q_EMIT pluginProjectRemoved(project->baseDir(), project->name());
+
+    // update actions, e.g. the close all stuff needs this
+    updateActions();
 }
 
 QString KateProjectPluginView::currentWord() const
@@ -714,8 +787,11 @@ void KateProjectPluginView::slotUpdateStatus(bool visible)
         return;
     }
 
-    if (auto widget = m_stackedgitViews->currentWidget()) {
-        static_cast<GitWidget *>(widget)->getStatus();
+    if (auto widget = static_cast<GitWidget *>(m_stackedGitViews->currentWidget())) {
+        // To support separate-git-dir always use dotGitPath
+        // We need to add the path every time again because it's always a different file
+        m_gitChangedWatcher.addPath(widget->dotGitPath() + QStringLiteral(".git/index"));
+        widget->updateStatus();
     }
 }
 
@@ -729,11 +805,7 @@ void KateProjectPluginView::openDirectoryOrProject()
 
     // switch to this project if there
     if (auto project = m_plugin->projectForDir(dir, true)) {
-        int index = m_projectsCombo->findData(project->fileName());
-        if (index >= 0) {
-            m_projectsCombo->setCurrentIndex(index);
-        }
-        mainWindow()->showToolView(m_toolView);
+        slotActivateProject(project);
     }
 }
 
@@ -804,6 +876,17 @@ void KateProjectPluginView::updateActions()
     m_projectNextAction->setEnabled(projectActive);
     m_projectGotoIndexAction->setEnabled(projectActive);
     m_projectCloseAction->setEnabled(projectActive);
+    m_projectCloseAllAction->setEnabled(m_projectsCombo->count() > 0);
+    m_projectCloseWithoutDocumentsAction->setEnabled(m_projectsCombo->count() > 0);
+}
+
+void KateProjectPluginView::slotActivateProject(KateProject *project)
+{
+    const int index = m_projectsCombo->findData(project->fileName());
+    if (index >= 0) {
+        m_projectsCombo->setCurrentIndex(index);
+        mainWindow()->showToolView(m_toolView);
+    }
 }
 
 #include "kateprojectpluginview.moc"
