@@ -95,14 +95,15 @@ QVector<KateSearchMatch> SearchDiskFiles::searchSingleLineRegExp(QFile &file)
 
             // remember match
             const int endColumn = column + match.capturedLength();
-            const int preContextStart = qMax(0, column - MatchModel::PreContextLen);
+            const auto [preContextStart, postContextLen] = MatchModel::contextLengths(line.size(), column, endColumn);
             const QString preContext = line.mid(preContextStart, column - preContextStart);
-            const QString postContext = line.mid(endColumn, MatchModel::PostContextLen);
+            const QString postContext = line.mid(endColumn, postContextLen);
             matches.push_back(KateSearchMatch{preContext,
                                               match.captured(),
                                               postContext,
                                               QString(),
                                               KTextEditor::Range{currentLineNumber, column, currentLineNumber, column + match.capturedLength()},
+                                              true,
                                               true});
 
             // advance match column
@@ -185,7 +186,7 @@ QVector<KateSearchMatch> SearchDiskFiles::searchMultiLineRegExp(QFile &file)
         QString postContext = fullDoc.mid(column + match.captured().length(), MatchModel::PostContextLen);
 
         matches.push_back(
-            KateSearchMatch{preContext, match.captured(), postContext, QString(), KTextEditor::Range{line, startColumn, endLine, endColumn}, true});
+            KateSearchMatch{preContext, match.captured(), postContext, QString(), KTextEditor::Range{line, startColumn, endLine, endColumn}, true, true});
 
         match = tmpRegExp.match(fullDoc, column + match.capturedLength());
         column = match.capturedStart();
