@@ -1,25 +1,12 @@
 /*  This file is part of the Kate project.
  *  Based on the snippet plugin from KDevelop 4.
  *
- *  Copyright (C) 2007 Robert Gruber <rgruber@users.sourceforge.net>
- *  Copyright (C) 2010 Milian Wolff <mail@milianw.de>
- *  Copyright (C) 2012 Christoph Cullmann <cullmann@kde.org>
- *  Copyright (C) 2014 Sven Brauch <svenbrauch@gmail.com>
+ *  SPDX-FileCopyrightText: 2007 Robert Gruber <rgruber@users.sourceforge.net>
+ *  SPDX-FileCopyrightText: 2010 Milian Wolff <mail@milianw.de>
+ *  SPDX-FileCopyrightText: 2012 Christoph Cullmann <cullmann@kde.org>
+ *  SPDX-FileCopyrightText: 2014 Sven Brauch <svenbrauch@gmail.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License as published by the Free Software Foundation; either
- *  version 2 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "editrepository.h"
@@ -29,13 +16,17 @@
 #include <ktexteditor/document.h>
 #include <ktexteditor/editor.h>
 
+#include <KConfigGroup>
 #include <KLocalizedString>
+#include <KSharedConfig>
 #include <KStandardGuiItem>
 #include <KUser>
 
 #include <QDialogButtonBox>
 #include <QPushButton>
 
+static const char s_configFile[] = "kate-snippetsrc";
+
 EditRepository::EditRepository(SnippetRepository *repository, QWidget *parent)
     : QDialog(parent)
     , Ui::EditRepositoryBase()
@@ -97,6 +88,13 @@ EditRepository::EditRepository(SnippetRepository *repository, QWidget *parent)
     validate();
     updateFileTypes();
     repoNameEdit->setFocus();
+
+    KSharedConfigPtr config = KSharedConfig::openConfig(QLatin1String(s_configFile));
+    KConfigGroup group = config->group("General");
+    const QSize savedSize = group.readEntry("Size", QSize());
+    if (savedSize.isValid()) {
+        resize(savedSize);
+    }
 }
 
 void EditRepository::validate()
@@ -126,6 +124,11 @@ void EditRepository::save()
     m_repo->save();
 
     setWindowTitle(i18n("Edit Snippet Repository %1", m_repo->text()));
+
+    KSharedConfigPtr config = KSharedConfig::openConfig(QLatin1String(s_configFile));
+    KConfigGroup group = config->group("General");
+    group.writeEntry("Size", size());
+    group.sync();
 }
 
 void EditRepository::updateFileTypes()
